;
;	diseqc_2x16.asm
;       DISEQC signal monitor                   N0QBH   2012
;	For PIC16F628A micro
;
;	16x2 LCD display for up to 2 packets
;	Switch toggles run/stop, hold for clear
;	Cursor on/running, off/stopped
;	Zero crossing 22 kHz tone detection
;	Repeat packet(*)doesn't over write next
;	Serial data out @ 9k6 
;	1 - 6 byte packets
;	4 Mhz Internal OSC
;	11/10/12			
;	Ver	2.16S				
;------------------------------------------------------------
        LIST    P=16F628A
	ERRORLEVEL	-302
        include <P16F628A.INC>
        __CONFIG        h'3d30'
;-------------------------------------------------------------
;	Updated port map for rs232 ver
;               Port Set-up
;           0   1   2   3   4   5   6   7       bit
; port  A   Out	In  In  Out In  Rst E   RS
;       B   Out In  In  Out LCD LCD LCD LCD
;------------------------------------------------------------
;
	ORG	0			; start code
	goto	main
;
;------------------------------------------------------------
        cblock  0x20
;
	buffer_0		;(20h) buffer 
        buffer_1                ;
        buffer_2                ;
	buffer_3		;
	buffer_4		;
	buffer_5		;
	buffer_6		;up to 6 bytes
;
	lcd_count		;LCD util counter
        bit_count              	;bit counter 
        zcd_count             	;zero crossings cnt
	delay_count		;delays
	lull_count		;
;
        byte_in                 ;data byte rx'd 
	bits_cnt		;another bit counter
	byte_count		;num converted bytes
;
	B_0			;memory store offset
        B_1                	;(30h)buffer 
        B_2                	;buffer for parity calc
	B_3			;hex to LCD buffer
;
	lcd_data		;LCD display data 
	send_reg		;serial out buffer
	alt_buffer		; 
        FLAGS                   ;flags 
       	alternator              ;(37h)loop counter
	endc
;------------------------------------------------------------
;
#define	BIT_FLAG	FLAGS,0		;0/1
#define	PY_FLAG		FLAGS,1		;parity 1/0
#define	FR_FLAG		FLAGS,2		;1st byte y/n
#define	ST_FLAG		FLAGS,3		;last packet repeat
;
#define	SW_COM		PORTA,4		;switch input (mapped from B:1)
#define E_LINE          PORTA,6         ;LCD E (mapped from B:3)
#define RS_LINE         PORTA,7         ;LCD RS (mapped from B:2)
;
DS_START	equ	0x40		;start of data store
;
CR		equ	0x0a		;CR
LF		equ	0x0d		;LF
;
US_125		equ	.130		;125 uS
MS_15		equ	.60		;15 mS
MS_60		equ	.240		;60 mS
;
;------------------------------------------------------------
header_message
        addwf   PCL,F
        dt      "DISEQC DEC V2.16",0
;
hex2ascii
        addwf   PCL,F
        dt      "0123456789ABCDEF"
;
;------------------------------------------------------------
;
main
        movlw   0x05			;single compare module
        movwf   CMCON                   ;no ext output
        clrf    PORTA
        clrf    PORTB
;
        bsf     STATUS,RP0              ;switch to bank 1
        movlw   b'00010110'             ;port A in/outputs
        movwf   TRISA			;A:1,2 comp in
        movlw   b'00000110'             ;port B in/outputs
        movwf   TRISB			;
;
	clrwdt				;clear wdt
        movlw   b'00100100'
	movwf	TXSTA			;tx enable, BRGH high
	movlw	d'25'			;9k6 data rate
	movwf	SPBRG			;async setup
;
	movlw	b'00001000'
        movwf   OPTION_REG              ;TMR0/1, RB0 falling
        movlw   b'00000000'             ;WPUs on
        movwf   PIE1                    ;perif int setup
;
        bcf     STATUS,RP0              ;bank 0
;
        movlw   b'10000000'
	movwf	RCSTA			;async serial enabled
        movlw   b'00000000'             ;
	movwf	PIR1			;clear any perifs
	movlw	b'0000000'
	movwf	T1CON			;T1/1, off
;
        movlw   b'00000100'             ;post/1, pre/1, on
        movwf   T2CON                   ;timer 2
        movlw   b'00000000'             ;
	movwf	INTCON			;no interrupts
;
	call	delay_60mS		;let things settle
	call	init_lcd
;
	call	send_header_to_LCD	;show life
	call	wrap_serial		;CR/LF
	clrf	alternator		;
	clrf	FLAGS			;
;
;------------------------------------------------------------
;	Prep for new capture
;
start_here
	movlw	DS_START		;start store loc
	movwf	B_1			;point to store loc
;
	movlw	0x20			;(1)
	movwf	FSR			;(1)reset FSR pointer
house_clean_loop
	clrf	INDF			;(1)clear named registers
	incf	FSR,W			;(1)
	sublw	bit_count 		;(1)
	bnc	tone_detect		;(2,3)
	incf	FSR,F			;(1)
	goto	house_clean_loop	;(2)
;
;	Cleanup above  -  Data detect below
;
tone_detect
	clrf	zcd_count		;(1)reset
	clrf	lull_count		;(1)ditto
	movf	CMCON,F			;(1)read & reset
	bcf	PIR1,CMIF		;(1)clear compare flag
;	
holdoff_TO
	movlw	.156			;(1)100 uS to rollover Time Out
	movwf	TMR0			;(1)
	bcf 	INTCON,T0IF		;(1)insure reset
;
check_ints
	btfsc	INTCON,T0IF		;(1/2)timeout?
	goto	zcd_lull		;(2)yes, tone done
	btfss	PIR1,CMIF		;(1/2)comparitor interrupt?
	goto	check_ints		;(2)loop
;
comp_handler
	movf	CMCON,F			;(1)read & reset
	bcf	PIR1,CMIF		;(1)
	incf	zcd_count,F		;(1)update count
	goto	holdoff_TO		;(2)loop back
;
zcd_lull
	movf	zcd_count,F		;test for too short
	bnz	stash_loot		;
	incf	lull_count,F		;no ints counted +1
	btfss	lull_count,5		;>32? (3.2mS)
	goto	holdoff_TO		;no
	btfss	SW_COM			;switch pressed?
	goto	cease_op		;yes, stop
	goto	interprit		;no, exit capture 
;
stash_loot
	movfw	zcd_count		;
	sublw	.16			;
	skpnc				;
	goto	tone_detect		;<17 xings
;
check_cycles
	movlw	.27			;test <= 26 xings
	subwf	zcd_count,W		;
	skpnc				;
	goto	test_4_0		;too long for a 1, test for 0
 	setc				;call it a one
	goto	roll_in_bit		;exit & store
;
test_4_0
	movlw	.36			;test => 36 xings
	subwf	zcd_count,W		;
	skpc
	goto	tone_detect		;too few cycles for a 0
	movlw	.52			;<52 cycles?
	subwf	zcd_count,W		;
	skpnc
	goto	tone_detect		;too long
	clrc				;call it a zero
roll_in_bit
	rlf	buffer_0,F		;roll carry into buffer_0-6
	rlf	buffer_1,F		;we're looking for
	rlf	buffer_2,F		;up to 6 bytes 
	rlf	buffer_3,F		;and parity (54 bits)
	rlf	buffer_4,F		;
	rlf	buffer_5,F		;
	rlf	buffer_6,F		;last to fill	
	incf	bit_count,F		;count bits
	movfw	bit_count		;count to W
	xorlw	.4			;
	bz	first_frame
	movfw	bit_count		;count to W
	xorlw	.54			;
	bz	interprit		;end of memory - try to show
	goto	tone_detect		;back to get next tone
;
first_frame
	movfw	buffer_0		;1st nybble
	andlw	b'00001111'		;
	xorlw	0x0e			;is it E?
	skpnz
	goto	tone_detect		;yes, back to get next tone
	goto	start_here		;no, bail and start over
;
;------------------------------------------------------------
;	Shift bits to end of buffers and count them
;
interprit
	movfw	bit_count		;
	sublw	.8			;
	skpnc				;>8 ?
	goto	start_here		;no
	movfw	bit_count		;yes, check max
	sublw	.54			;
	bnc	start_here		;too long
	movfw	bit_count		;
	movwf	B_0			;working bit count
;	
roll2end_loop
	call	barrel_shift		;
;
	incf	B_0,F			;rotate buffers til 
	movfw	B_0			;aligned with end
	xorlw	.56			;at end? (7x8bits)
	skpz				;
	goto	roll2end_loop		;no
;
	clrf	byte_count		;start at 0
r2e_loop
	movlw	.9			;
	subwf	bit_count,W		;
	skpc				;
	goto	start_here		;no match
	skpz	
	goto	not_yetA		;
	incf	byte_count,F		;last byte end match
	movfw	byte_count		;
	movwf	B_0			;store byte_count
	goto	check_remove_parity	;do next
;
not_yetA
	movlw	.9			;
	subwf	bit_count,F		;
	incf	byte_count,W		;no, inc byte_count
	xorlw	.7			;test for over run
	bz	start_here		;epic fail
	incf	byte_count,F		;
	goto	r2e_loop		;loop
;
;	Do parity check on data and compare to received
;
check_remove_parity
	movlw	.8
	movwf 	bits_cnt		;8 loops
chre_par_loop
	call	barrel_shift		;
	rlf	byte_in,F		;shift to this byte	
	decfsz	bits_cnt,F		;8 loops
	goto	chre_par_loop
;
	movfw	byte_in			;new byte to W
	movwf	B_2			;W to parity calc byte
	call	parity_calc		;result in PY_FLAG
;
	call	barrel_shift		;rotate parity bit out
	skpc				;test carry
	goto	ris0			;rxd parity bit =0
	btfss	PY_FLAG			;=1, rxd & calc high?
	goto	start_here		;no, exit
	goto	parityOK		;flag set, match
;
ris0
	btfsc	PY_FLAG			;=0, rxd & calc low?
	goto	start_here		;no
;
parityOK
	movfw	B_1			;pointer for data store	
	movwf	FSR			;
	movfw	byte_in			;recovered byte
	movwf	INDF			;store at offset
	incf	B_1,F			;B_1 +1
	decfsz	byte_count,F		;count down the bytes
	goto	check_remove_parity	;loop back for more
;
	movfw	B_0			;
	movwf	byte_count		;restore orig byte_count
;
;------------------------------------------------------------
;	Checks for framing byte, picks line, displays data
;
check4frame
	movlw	DS_START		;
	movwf	B_1			;point to store loc
;
	movwf	FSR			;load FSR
	movfw	INDF			;load W	
	andlw	b'11110000'		;mask low nibble
	xorlw	b'11100000'		;test for framing byte
	skpz				;1st byte
	goto	start_here		;not framing byte
;
;	Saw framing byte, see if this byte is a copy of previous
;
mirror_mirror
	movfw	INDF			;pointed to new byte
	movwf	B_2			;to buffer2
	movfw	FSR			;FSR to W
	addlw	.10			;mirror regs loc
	movwf	FSR			;W to FSR +10
	movfw	INDF			;mirrored value to W
	xorwf	B_2,W			;compare to buffer2 
	skpz				;test for match
	goto	packet2LCD		;mismatch, display it
	decfsz	B_0,F			;match, dec byte counter
	goto	loop2mirror		;go back to next new byte
show_repeat
	movlw	0x8E			;line #1 15th char
	btfss	alternator,0		;point to prev line
	movlw	0xCE			;line #2 15th char
	movwf	alt_buffer		;store loc 
	call	liter_c			;
	movlw	"*"			;
	movwf	lcd_data		;
	movwf	send_reg		;serial
	call	liter_d			;display star at lines end
	call	do_serial_send		;
	bsf	ST_FLAG			;set flag
	goto	start_here		;start at beginning
;
loop2mirror
	movlw	.9			;
	subwf	FSR,F			;back to next byte		
	goto	mirror_mirror		;	
;
;	Uses bit 0 of a reg to alternate lines
;
packet2LCD
	movlw	0x80			;preload line #1
	btfsc	alternator,0		;which line?
	movlw	0xC0			;line #2
	movwf	alt_buffer		;store loc 
	call	liter_c			;
	movlw	.16			;number of spaces
	call	LCD_line_stuff		;clear 16 more locs
	movfw	alt_buffer		;
	call	liter_c			;
	incf	alternator,F		;increment alternator	
;
	btfsc	ST_FLAG			;test flag
	call	wrap_serial		;flag set, send CR/LF
;
looptheloop
	movfw	B_1
	movwf	FSR			;W to indr offset
	movfw	INDF			;load byte to W
;	
;	Convert to ascii hex and display
;
	movwf	B_3			;W to buffer
	swapf	B_3,W			;
	andlw	b'00001111'		;mask upper nibble
	call	hex2ascii		;lookup
	movwf	lcd_data		;put returned value in
	movwf	send_reg		;serial
	call	liter_d			;send msb to LCD
	call	do_serial_send
;
	movfw	B_3			;do lsn
	andlw	b'00001111'		;mask upper nibble
	call	hex2ascii		;
	movwf	lcd_data		;
	movwf	send_reg		;serial
	call	liter_d			;send msb to LCD
	call	do_serial_send
;
	movlw	" "			;insert space
	movwf	lcd_data		;
	movwf	send_reg		;serial
	call	liter_d			;send msb to LCD
	call	do_serial_send
;
	movfw	FSR			;present offset
	addlw	.10			;add 10
	movwf	FSR			;mirror offset (present +10)
	movfw	B_3			;store byte in mirror
	movwf	INDF			;
	incf	B_1,F			;inc pointer
	decfsz	byte_count,F		;
	goto	looptheloop		;get next byte
	call	wrap_serial		;CR/LF
;
	goto	start_here		;start fresh
;
;------------------------------------------------------------
;
cease_op				;switch low
        movlw   0x0c                    ;cursor off
        call    liter_c			;indicate stopped
	clrf	buffer_6		;use as counter
	call	delay_15mS		;changed from 60
co_loop	
	btfsc	SW_COM			;wait til hi agn
	goto	co_loop2		;went high (open)
	call	delay_15mS		;
	incf	buffer_6,W		;
	sublw	.134			;
	bnc	scrnclr			;
	incf	buffer_6,F		;count how long pressed
	goto	co_loop			;
scrnclr	
	call	clear_LCD_display	;clear screen	
	clrf	alternator		;reset display loc
;
co_loop2
	call	delay_15mS		;
	btfsc	SW_COM			;wait for low
	goto	co_loop2		;still high
	call	delay_15mS		;
co_loop3
	btfss	SW_COM			;went low, wait for hi
	goto	co_loop3		;high
	movlw   0x0e                    ;low, cursor on
        call    liter_c			;indicate running
	goto	start_here		;start fresh	
;
;------------------------------------------------------------
;
;       Subroutines area
;
;------------------------------------------------------------
;
barrel_shift
	clrc				;clear carry
	rlf	buffer_0,F		;
	rlf	buffer_1,F		;
	rlf	buffer_2,F		;
	rlf	buffer_3,F		;
	rlf	buffer_4,F		;
	rlf	buffer_5,F		;
	rlf	buffer_6,F		;last to fill
	return
;
;------------------------------------------------------------
;
;This routine will leave the parity of X in X.0 
;while blenderizing most of the rest of X 
;
parity_calc
	bcf	PY_FLAG		;clear flag
        swapf   B_2, W		;
        xorwf   B_2, F		;
        rrf     B_2, W 		;
        xorwf   B_2, F 		;
        btfsc   B_2, 2 		;if the parity of (a,b,e,f) is 0,
        incf    B_2, F 		;otherwise, the parity of (a,b,e,f) is 1,

	btfss	B_2,0		;test bit0
	bsf	PY_FLAG		;invert for odd parity
	return	
;
;------------------------------------------------------------
;
send_header_to_LCD
        movlw   0x80			;LINE_1
        call    liter_c                 ;starting point on line 1
	clrf	lcd_count		;reset index
header_loop
	movfw	lcd_count		;load W
	call	header_message		;
	movwf	lcd_data		;
	movwf	send_reg		;serial out
	movf	lcd_data,F		;test for 0
	skpnz				;
	return				;saw a zero, exit
;
	call	liter_d			;send to LCD
	incf	lcd_count,F		;inc offset
	call	do_serial_send		;send serially
	goto	header_loop		;loop til 0
;
;------------------------------------------------------------
;	Sends serial data w/USART module
;
do_serial_send
        btfss   PIR1,TXIF		;wait for clear register
        goto    do_serial_send
	movf	send_reg,W		;
	movwf	TXREG			;send it
	return
;
wrap_serial
	movlw	CR
	movwf	send_reg		;
	call	do_serial_send
	movlw	LF
	movwf	send_reg		;
	call	do_serial_send
	bcf	ST_FLAG			;clear flag
	return
;
;------------------------------------------------------------
;               LCD Routines
;------------------------------------------------------------
;
init_lcd
        bcf     E_LINE                  ;E line low
        bcf     RS_LINE                 ;RS low, control
        call    delay_15mS              ;delay 15 mS
        movlw   0x30                    ;8-bit, 5X7 mode
        movwf   lcd_data
        call    flipbit
        call    pulse
	call	delay_15mS
        movlw   0x20                    ;4-bit, 5x7 mode
        movwf   lcd_data
        call    flipbit
        call    pulse
	call	delay_15mS
        movlw   0x28                    ;4-bit, 5x7 mode
        call    liter_c                 ;send both nybbles
        movlw   0x0e                    ;display on, cursor on
        call    liter_c			;no blink
        movlw   0x06                    ;increment mode
        call    liter_c
        movlw   0x01                    ;clear display
        call    liter_c
 	call	delay_15mS
        return
;
;------------------------------------------------------------
;	Line_clear does whole line 
;	Line_stuff clears W number of chars
;
LCD_line_clear
	movlw	.16			;countdown from 16
LCD_line_stuff	
	movwf	lcd_count		;countdown from W 
line_clear_loop	
	movlw	" "
	movwf	lcd_data
	call	liter_d
	decfsz	lcd_count,F
	goto	line_clear_loop
	return
;       
;------------------------------------------------------------
;		sends control byte to LCD
liter_c					
        movwf   lcd_data                ;store w in lcd_data
	bcf	RS_LINE			;control
        call    delay_125uS
        call    flipbit                 ;hi nibble to PORTB
	call	pulse
        swapf   lcd_data,F              ;swap MS and LS nybbles
        call    flipbit                 ;output what was LS nybble
	call	pulse
	return
;
;------------------------------------------------------------
;		sends lcd_data to LCD
liter_d					
	bsf	RS_LINE			;data
        call    delay_125uS
        call    flipbit                 ;hi nibble to PORTB
	call	pulse
        swapf   lcd_data,F              ;swap MS and LS nybbles
        call    flipbit                 ;output what was LS nybble
	call	pulse
	return
;
;------------------------------------------------------------
;		clears upper nibble of PORTB then writes
;		upper nibble of lcd_data to PORTB
flipbit
	movlw	0x0f			;
	andwf	PORTB,F			;
	movfw	lcd_data		;
	andlw	0xf0			;
	iorwf 	PORTB,F			;
	return				
;
;------------------------------------------------------------
;
pulse
        bsf     E_LINE                  ;pulse E line
        nop                             ;delay
        bcf     E_LINE
        call    delay_125uS             ;delay 125 uS
        return
;
;------------------------------------------------------------
;
clear_LCD_display
        movlw   0x01                    ;clear display
        call    liter_c                 ;send to display
        call    delay_15mS              ;allow 15 mS to clear
        return
;
;------------------------------------------------------------
;
delay_125uS
	movlw	.1
	movwf	delay_count		;
	movlw	US_125
	movwf	TMR2
	goto	T2_delay_loop
;
delay_15mS
	movlw	MS_15
	movwf	delay_count		;
	clrf	TMR2
	goto	T2_delay_loop
;
delay_60mS
	movlw	MS_60	
	movwf	delay_count		;
	clrf	TMR2
;
T2_delay_loop
	bcf	PIR1,TMR2IF		;256 uS per rollover
T2_loopA
	btfss	PIR1,TMR2IF
	goto	T2_loopA
	decfsz	delay_count,F		;
	goto	T2_delay_loop
	return
;	
;------------------------------------------------------------
        END
;------------------------------------------------------------
;
;Copyright (c) 2012, Mike Berg aka N0QBH
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, 
;are permitted provided that the following conditions are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright notice, 
;this list of conditions and the following disclaimer in the documentation 
;and/or other materials provided with the distribution. 
;Neither the name of the <ringolake.com> nor the names of its contributors 
;may be used to endorse or promote products derived from this software without 
;specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
;AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
;THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
;PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
;BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
;SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
;INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
;ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;THE POSSIBILITY OF SUCH DAMAGE.
;------------------------------------------------------------
